@@ -12,6 +12,8 @@
 #include <linux/version.h>
 #include <unistd.h>
 
+#include "../../arch/all/init.h"
+
 // see: https://github.com/giampaolo/psutil/issues/659
 #ifdef PSUTIL_ETHTOOL_MISSING_TYPES
     #include <linux/types.h>
@@ -25,8 +27,6 @@
 #define _LINUX_SYSINFO_H
 #include <linux/ethtool.h>
 
-#include "../../_psutil_common.h"
-
 
 // * defined in linux/ethtool.h but not always available (e.g. Android)
 // * #ifdef check needed for old kernels, see:
