@@ -16,25 +16,33 @@
 #include <sys/sysctl.h>
 #include <mach/mach.h>
 
-#include "../../_psutil_posix.h"
+#include "../../arch/all/init.h"
 
 
 static int
 psutil_sys_vminfo(vm_statistics64_t vmstat) {
     kern_return_t ret;
-    unsigned int count = HOST_VM_INFO64_COUNT;
-    mach_port_t mport = mach_host_self();
+    mach_msg_type_number_t count = HOST_VM_INFO64_COUNT;
+    mach_port_t mport;
+
+    mport = mach_host_self();
+    if (mport == MACH_PORT_NULL) {
+        PyErr_SetString(PyExc_RuntimeError,
+                        "mach_host_self() returned MACH_PORT_NULL");
+        return 1;
+    }
 
     ret = host_statistics64(mport, HOST_VM_INFO64, (host_info64_t)vmstat, &count);
+    mach_port_deallocate(mach_task_self(), mport);
     if (ret != KERN_SUCCESS) {
         PyErr_Format(
             PyExc_RuntimeError,
-            "host_statistics(HOST_VM_INFO) syscall failed: %s",
-            mach_error_string(ret));
-        return 0;
+            "host_statistics64(HOST_VM_INFO64) syscall failed: %s",
+            mach_error_string(ret)
+        );
+        return 1;
     }
-    mach_port_deallocate(mach_task_self(), mport);
-    return 1;
+    return 0;
 }
 
 
@@ -48,7 +56,6 @@ PyObject *
 psutil_virtual_mem(PyObject *self, PyObject *args) {
     int      mib[2];
     uint64_t total;
-    size_t   len = sizeof(total);
     vm_statistics64_data_t vm;
     long pagesize = psutil_getpagesize();
     // physical mem
@@ -56,22 +63,16 @@ psutil_virtual_mem(PyObject *self, PyObject *args) {
     mib[1] = HW_MEMSIZE;
 
     // This is also available as sysctlbyname("hw.memsize").
-    if (sysctl(mib, 2, &total, &len, NULL, 0)) {
-        if (errno != 0)
-            PyErr_SetFromErrno(PyExc_OSError);
-        else
-            PyErr_Format(
-                PyExc_RuntimeError, "sysctl(HW_MEMSIZE) syscall failed");
+    if (psutil_sysctl(mib, 2, &total, sizeof(total)) != 0)
         return NULL;
-    }
 
     // vm
-    if (!psutil_sys_vminfo(&vm))
+    if (psutil_sys_vminfo(&vm) != 0)
         return NULL;
 
     return Py_BuildValue(
         "KKKKKK",
-        total,
+        (unsigned long long) total,
         (unsigned long long) vm.active_count * pagesize,  // active
         (unsigned long long) vm.inactive_count * pagesize,  // inactive
         (unsigned long long) vm.wire_count * pagesize,  // wired
@@ -87,30 +88,24 @@ psutil_virtual_mem(PyObject *self, PyObject *args) {
 PyObject *
 psutil_swap_mem(PyObject *self, PyObject *args) {
     int mib[2];
-    size_t size;
     struct xsw_usage totals;
     vm_statistics64_data_t  vmstat;
     long pagesize = psutil_getpagesize();
 
     mib[0] = CTL_VM;
     mib[1] = VM_SWAPUSAGE;
-    size = sizeof(totals);
-    if (sysctl(mib, 2, &totals, &size, NULL, 0) == -1) {
-        if (errno != 0)
-            PyErr_SetFromErrno(PyExc_OSError);
-        else
-            PyErr_Format(
-                PyExc_RuntimeError, "sysctl(VM_SWAPUSAGE) syscall failed");
-        return NULL;
-    }
-    if (!psutil_sys_vminfo(&vmstat))
+
+    if (psutil_sysctl(mib, 2, &totals, sizeof(totals)) != 0)
+        return psutil_PyErr_SetFromOSErrnoWithSyscall("sysctl(HW_CPU_FREQ)");
+
+    if (psutil_sys_vminfo(&vmstat) != 0)
         return NULL;
 
     return Py_BuildValue(
-        "LLLKK",
-        totals.xsu_total,
-        totals.xsu_used,
-        totals.xsu_avail,
-        (unsigned long long)vmstat.pageins * pagesize,
-        (unsigned long long)vmstat.pageouts * pagesize);
+        "KKKKK",
+        (unsigned long long) totals.xsu_total,
+        (unsigned long long) totals.xsu_used,
+        (unsigned long long) totals.xsu_avail,
+        (unsigned long long) vmstat.pageins * pagesize,
+        (unsigned long long) vmstat.pageouts * pagesize);
 }
